/*
 * Gv4l's Transcode related calls
 * Watch, Record, Stop, Transcode Output functions
 * 
 * Author:	Todor T. Zviskov (a.k.a. Warder) - http://warder.ath.cx:81/
 * License:	GPL-2 - http://www.gnu.org/licenses/gpl.txt
 *
 * (C)2002-2004 Todor T. Zviskov.
 */


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <gnome.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "gv4l.h"
#include "interface.h"
#include "support.h"

extern GtkWidget *gv4l;		//is initialized in main.c
extern GtkWidget *pref;
extern char config_file[250];	//config file variable from main.c
extern gboolean record;		//record on startup var from main.c
extern gint visible[3];		//window visibility
extern gchar v4l_info[9][255];	//v4l device info
GtkWidget *transcode_output;

#define BUFFER_SIZE 8192



//watch function
void on_watch_clicked (GtkButton * button, gpointer user_data)
{
	//define variables
	FILE *outFile;

	gchar vdev_val[250];
	gchar adev_val[250];
	gchar p_size[250] = "-g ";
	gchar fps_val[250] = "-f ";
	gchar src_val[250], vsrc_val[250], temp[250], *pos;
	gint i = 0;
	gchar chan_val[250];
	gchar norm_val[10];
	gboolean deinter;
	gboolean prev_val;
	gboolean yuv_val;
	//lookup widgets
	GtkWidget *vdev = lookup_widget (GTK_WIDGET (button), "vdev_val");
	GtkWidget *adev = lookup_widget (GTK_WIDGET (button), "adev_val");
	GtkWidget *size = lookup_widget (GTK_WIDGET (button), "p_size");
	GtkWidget *fps = lookup_widget (GTK_WIDGET (button), "fps_val");
	GtkWidget *src = lookup_widget (GTK_WIDGET (button), "src_val");
	GtkWidget *chan = lookup_widget (GTK_WIDGET (button), "chan_val");
	GtkWidget *prev = lookup_widget (GTK_WIDGET (button), "prev_val");
	GtkWidget *inter = lookup_widget (GTK_WIDGET (button), "de_inter");
	GtkWidget *yuv = lookup_widget (GTK_WIDGET (button), "yuv_val");
	GtkWidget *norm = lookup_widget (GTK_WIDGET (button), "norm_val");

	GtkWidget *status = lookup_widget (GTK_WIDGET (button), "status");
	GtkStyle *style = gtk_style_new ();
	GtkStyle *style0 = gtk_style_new ();
	GdkColor blue = { 0, 0, 0, 65335 };
	GdkColor green = { 0, 0, 35335, 0 };

	strncpy (vdev_val, gtk_entry_get_text (GTK_ENTRY (vdev)), 250);
	strncpy (adev_val, gtk_entry_get_text (GTK_ENTRY (adev)), 250);

	//check if pic size is set
	strcat (p_size, gtk_entry_get_text (GTK_ENTRY (size)));
	if (strcmp (p_size, "-g ") == 0)
		strcpy (p_size, " ");

	//check if fps is set
	strcat (fps_val, gtk_entry_get_text (GTK_ENTRY (fps)));
	if (strcmp (fps_val, "-f ") == 0)
		strcpy (fps_val, " ");

	//check if preview is set
	strncpy (vsrc_val, gtk_entry_get_text (GTK_ENTRY (src)), 250);
	strncpy (chan_val, gtk_entry_get_text (GTK_ENTRY (chan)), 250);
	strncpy (norm_val, gtk_entry_get_text (GTK_ENTRY (norm)), 10);
	prev_val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prev));
	deinter = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (inter));
	yuv_val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (yuv));

	//error if no video dev set
	if (strlen (vdev_val) == 0)
	{
		GtkWidget *novid;
		novid = create_novid ();
		gtk_widget_show (novid);
		gtk_window_set_transient_for (GTK_WINDOW (novid), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	//error if no audio dev set
	if (strlen (adev_val) == 0)
	{
		GtkWidget *noaud;
		noaud = create_noaud ();
		gtk_widget_show (noaud);
		gtk_window_set_transient_for (GTK_WINDOW (noaud), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	//error if no source set
	if (strlen (vsrc_val) == 0)
	{
		GtkWidget *nosrc;
		nosrc = create_nosrc ();
		gtk_widget_show (nosrc);
		gtk_window_set_transient_for (GTK_WINDOW (nosrc), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	for (i = 0; i < 9; i++)
	{
		sprintf (temp, "=%s\0", vsrc_val);
		if (strstr (v4l_info[i], vdev_val) && strstr (v4l_info[i], temp))
		{
			pos = strstr (v4l_info[i], temp) - 1;
			strncpy (src_val, pos, 1);
			src_val[1] = '\0';
		}
	}

	//stop transcode if running
	system ("killall -2 transcode 2>/dev/null || echo Done > /tmp/gv4l.temp");
	system ("killall -9 gv4l-sleep.sh 2>/dev/null");

	//wait until scripts are done
	do
		system ("sleep 0.5 2>/dev/null || sleep 0,5");
	while (fopen ("/tmp/gv4l.temp", "r") == NULL);
	system ("sleep 0.1 2>/dev/null || sleep 0,1");
	
	//remove temp files
	system ("rm -rf /tmp/gv4l*");

	//write new script
	outFile = fopen ("/tmp/gv4l.sh", "w");
	fprintf (outFile, "#!/bin/bash\n");
	//	fprintf (outFile, "v4lctl -c %s setnorm %s\n", vdev_val, norm_val); //set the TV standard (norm)
	fprintf (outFile,
		"transcode --progress_off -i %s -p %s -V -u 100 %s %s --import_v4l %s,\"%s\" %s %s %s\n",
		vdev_val, adev_val, p_size, fps_val, src_val, chan_val,
		yuv_val == TRUE ? "-x v4l=yuv422" : "",
		deinter == TRUE ? "-I 5" : "",
		prev_val == TRUE ? "-J preview" : "");
	fprintf (outFile, "echo -e '\n\n\'\n");
	fprintf (outFile, "echo Done > /tmp/gv4l.temp\n");
	fclose (outFile);

	//set the norm
	set_norm (vdev_val, norm_val);
	//start script
	system ("chmod +x /tmp/gv4l.sh");
	system ("/tmp/gv4l.sh >>/tmp/tr-output 2>&1 &");
	sleep (1);
	output_show ();

	//remove check file
	system ("rm -rf /tmp/trr");
	//create check file if transcode is running
	system ("ps -A|grep transcode >/tmp/trr || rm -rf /tmp/trr");

	//set new status
	style->text[GTK_STATE_NORMAL] = blue;
	// style->font = gdk_font_load("10x20");
	gtk_entry_set_text (GTK_ENTRY (status), "Playing Stream from Device...");
	gtk_widget_set_style (GTK_WIDGET (status), style);

	//reset status if transcode not running
	if (fopen ("/tmp/trr", "r") == NULL)
	{
		style0->text[GTK_STATE_NORMAL] = green;
		// style->font = gdk_font_load("10x20");
		gtk_entry_set_text (GTK_ENTRY (status), "Idle.");
		gtk_widget_set_style (GTK_WIDGET (status), style0);
	}
}

//call watch function from menu
void on_watch_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_watch_clicked ((GtkButton *) menuitem, menuitem);
}





//record function
void on_start_clicked (GtkButton * button, gpointer user_data)
{
	//define variables
	FILE *outFile;

	gchar vdev_val[250];
	gchar adev_val[250];
	gchar p_size[250] = "-g ";
	gchar fps_val[250] = "-f ";
	gchar cod_val[250];
	gchar src_val[250], vsrc_val[250], temp[250], *pos;
	gint i = 0;
	gchar chan_val[250];
	gchar file_val[250];
	gchar vbit_val[250] = "-w ";
	gchar abit_val[250] = "-b ";
	gchar agn_val[250] = "-s ";
	gchar norm_val[10];
	gboolean deinter;
	gchar rec_dur[250];
	int duration;
	gboolean prev_val;
	gboolean yuv_val;
	
	gchar split_val[20] = "--avi_limit ";
	
	//lookup widgets
	GtkWidget *vdev = lookup_widget (GTK_WIDGET (button), "vdev_val");
	GtkWidget *adev = lookup_widget (GTK_WIDGET (button), "adev_val");
	GtkWidget *size = lookup_widget (GTK_WIDGET (button), "p_size");
	GtkWidget *fps = lookup_widget (GTK_WIDGET (button), "fps_val");
	GtkWidget *cod = lookup_widget (GTK_WIDGET (button), "combo_entry1");
	GtkWidget *src = lookup_widget (GTK_WIDGET (button), "src_val");
	GtkWidget *chan = lookup_widget (GTK_WIDGET (button), "chan_val");
	GtkWidget *file = lookup_widget (GTK_WIDGET (button), "file_val");
	GtkWidget *vbit = lookup_widget (GTK_WIDGET (button), "vbit_val");
	GtkWidget *abit = lookup_widget (GTK_WIDGET (button), "abit_val");
	GtkWidget *agn = lookup_widget (GTK_WIDGET (button), "agn_val");
	GtkWidget *prev = lookup_widget (GTK_WIDGET (button), "prev_val");
	GtkWidget *rec = lookup_widget (GTK_WIDGET (button), "recTime");
	GtkWidget *inter = lookup_widget (GTK_WIDGET (button), "de_inter");
	GtkWidget *yuv = lookup_widget (GTK_WIDGET (button), "yuv_val");
	GtkWidget *norm = lookup_widget (GTK_WIDGET (button), "norm_val");
	
	GtkWidget *split = lookup_widget (pref, "split_val");

	GtkWidget *status = lookup_widget (GTK_WIDGET (button), "status");
	GtkStyle *style = gtk_style_new ();
	GtkStyle *style0 = gtk_style_new ();
	GdkColor red = { 0, 45335, 0, 0 };
	GdkColor green = { 0, 0, 35335, 0 };

	strncpy (vdev_val, gtk_entry_get_text (GTK_ENTRY (vdev)), 250);
	strncpy (adev_val, gtk_entry_get_text (GTK_ENTRY (adev)), 250);
	strncpy (rec_dur, gtk_entry_get_text (GTK_ENTRY (rec)), 250);
	//convert record dutation to integer value
	duration = atoi (rec_dur);

	//check if pic size is set
	strcat (p_size, gtk_entry_get_text (GTK_ENTRY (size)));
	if (strcmp (p_size, "-g ") == 0)
		strcpy (p_size, " ");

	//check if fps is set and use transcode internal FPS
	strcat (fps_val, gtk_entry_get_text (GTK_ENTRY (fps)));
	if (strcmp (fps_val, "-f ") == 0)
		strcpy (fps_val, " ");
	else if (strcmp (fps_val, "-f 23.976") == 0)
		strcpy (fps_val, "-f 0,1");
	else if (strcmp (fps_val, "-f 24") == 0)
		strcpy (fps_val, "-f 0,2");
	else if (strcmp (fps_val, "-f 25") == 0)
		strcpy (fps_val, "-f 0,3");
	else if (strcmp (fps_val, "-f 29.970") == 0)
		strcpy (fps_val, "-f 0,4");
	else if (strcmp (fps_val, "-f 30") == 0)
		strcpy (fps_val, "-f 0,5");
	else if (strcmp (fps_val, "-f 50") == 0)
		strcpy (fps_val, "-f 0,6");
	else if (strcmp (fps_val, "-f 59.940") == 0)
		strcpy (fps_val, "-f 0,7");
	else if (strcmp (fps_val, "-f 60") == 0)
		strcpy (fps_val, "-f 0,8");
	else if (strcmp (fps_val, "-f 1") == 0)
		strcpy (fps_val, "-f 0,9");
	else if (strcmp (fps_val, "-f 5") == 0)
		strcpy (fps_val, "-f 0,10");
	else if (strcmp (fps_val, "-f 10") == 0)
		strcpy (fps_val, "-f 0,11");
	else if (strcmp (fps_val, "-f 12") == 0)
		strcpy (fps_val, "-f 0,12");
	else if (strcmp (fps_val, "-f 15") == 0)
		strcpy (fps_val, "-f 0,13");

	strncpy (cod_val, gtk_entry_get_text (GTK_ENTRY (cod)), 250);
	strncpy (vsrc_val, gtk_entry_get_text (GTK_ENTRY (src)), 250);
	strncpy (chan_val, gtk_entry_get_text (GTK_ENTRY (chan)), 250);
	on_combo_entry1_focus_out_event ((GtkWidget *) button, NULL, button);
	strncpy (file_val, gtk_entry_get_text (GTK_ENTRY (file)), 250);
	strncpy (norm_val, gtk_entry_get_text (GTK_ENTRY (norm)), 10);

	//check if vid bitrate is set
	strcat (vbit_val, gtk_entry_get_text (GTK_ENTRY (vbit)));
	if (strcmp (vbit_val, "-w ") == 0)
		strcpy (vbit_val, " ");

	//check if audio dev is set
	strcat (abit_val, gtk_entry_get_text (GTK_ENTRY (abit)));
	if (strcmp (abit_val, "-b ") == 0)
		strcpy (abit_val, " ");

	//check if audio gain is set
	strcat (agn_val, gtk_entry_get_text (GTK_ENTRY (agn)));
	if (strcmp (agn_val, "-s ") == 0)
		strcpy (agn_val, " ");
	//see if de-interlace is set
	deinter = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (inter));


	prev_val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prev));
	yuv_val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (yuv));
	
	if (atoi (gtk_entry_get_text (GTK_ENTRY (split))) > 0)
		strcat (split_val, gtk_entry_get_text (GTK_ENTRY (split)));
	else
		split_val[0] = '\0';

	//stop transcode if running
	system ("killall -2 transcode 2>/dev/null || echo Done > /tmp/gv4l.temp");
	system ("killall -9 gv4l-sleep.sh 2>/dev/null");
	//wait until scripts are done
	do
		system ("sleep 0.5 2>/dev/null || sleep 0,5");
	while (fopen ("/tmp/gv4l.temp", "r") == NULL);
	system ("sleep 0.1 2>/dev/null || sleep 0,1");
	//remove temp files
	system ("rm -rf /tmp/gv4l*");


	// Check if all required fields are filled
	if (strlen (vdev_val) == 0)
	{
		GtkWidget *novid;
		novid = create_novid ();
		gtk_widget_show (novid);
		gtk_window_set_transient_for (GTK_WINDOW (novid), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	if (strlen (adev_val) == 0)
	{
		GtkWidget *noaud;
		noaud = create_noaud ();
		gtk_widget_show (noaud);
		gtk_window_set_transient_for (GTK_WINDOW (noaud), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	if (strlen (cod_val) == 0)
	{
		GtkWidget *nocod;
		nocod = create_nocod ();
		gtk_widget_show (nocod);
		gtk_window_set_transient_for (GTK_WINDOW (nocod), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	else if (strcmp (cod_val, "ffmpeg4") == 0)
		strcpy (cod_val, "ffmpeg -F mpeg4");
	else if (strcmp (cod_val, "ffmjpeg") == 0)
		strcpy (cod_val, "ffmpeg -F mjpeg");
	else if (strcmp (cod_val, "msmpeg4") == 0)
		strcpy (cod_val, "ffmpeg -F msmpeg4");
	else if (strcmp (cod_val, "msmpeg4v2") == 0)
		strcpy (cod_val, "ffmpeg -F msmpeg4v2");


	if (strlen (vsrc_val) == 0)
	{
		GtkWidget *nosrc;
		nosrc = create_nosrc ();
		gtk_widget_show (nosrc);
		gtk_window_set_transient_for (GTK_WINDOW (nosrc), GTK_WINDOW (gv4l));
		on_stop_clicked ((GtkButton *) button, button);
		return;
	}
	for (i = 0; i < 9; i++)
	{
		sprintf (temp, "=%s\0", vsrc_val);
		if (strstr (v4l_info[i], vdev_val) && strstr (v4l_info[i], temp))
		{
			pos = strstr (v4l_info[i], temp) - 1;
			strncpy (src_val, pos, 1);
			src_val[1] = '\0';
		}
	}
	// End Fields Check

	//set the audio recording source
	rec_src_set ((GtkEditable *) button, button);

	//write new script
	if (fopen (file_val, "r") == NULL)	//don't resume
	{
		outFile = fopen ("/tmp/gv4l.sh", "w");
		fprintf (outFile, "#!/bin/bash\n");
		//fprintf (outFile, "v4lctl -c %s setnorm %s\n", vdev_val, norm_val);		 //set the TV standard (norm)
		fprintf (outFile,
			 "transcode --progress_off -i %s -p %s -V -u 100 %s %s -y %s --import_v4l %s,\"%s\" %s -o \"%s\" %s %s %s %s %s %s",
			 vdev_val, adev_val, p_size, fps_val, cod_val, src_val,
			 chan_val, yuv_val == TRUE ? "-x v4l=yuv422" : "", file_val,
			 vbit_val, abit_val, agn_val, deinter == TRUE ? "-I 5" : "",
			 prev_val == TRUE ? "-J preview" : "", split_val);

		if (duration != 0)
		{
			fprintf (outFile, "&\n/tmp/gv4l-sleep.sh\n");
			fprintf (outFile, "\necho Done > /tmp/gv4l.temp\n");
		}
		else
			fprintf (outFile, "\necho Done > /tmp/gv4l.temp\n");

		fprintf (outFile, "echo -e '\n\n\'\n");
		fclose (outFile);

		if (duration != 0)
		{
			outFile = fopen ("/tmp/gv4l-sleep.sh", "w");
			fprintf (outFile, "#!/bin/bash\n");
			fprintf (outFile,
				 "sleep %dm\nkillall -2 transcode\nkillall -3 gv4l\nrm -rf /tmp/gv4l*\n",
				 duration);
			fclose (outFile);
		}

		//set the norm
		set_norm (vdev_val, norm_val);
		//run script
		system ("chmod +x /tmp/gv4l.sh");
		system ("chmod +x /tmp/gv4l-sleep.sh 2>/dev/null");
		system ("/tmp/gv4l.sh >>/tmp/tr-output 2>&1 &");
	}
	else				//resume recording
	{
		outFile = fopen ("/tmp/gv4l.sh", "w");
		fprintf (outFile, "#!/bin/bash\n");
		//fprintf (outFile, "v4lctl -c %s setnorm %s\n", vdev_val, norm_val);		 //set the TV standard (norm)
		fprintf (outFile,
			 "transcode --progress_off -i %s -p %s -V -u 100 %s %s -y %s --import_v4l %s,\"%s\" %s -o \"%s.append\" %s %s %s %s %s %s",
			 vdev_val, adev_val, p_size, fps_val, cod_val, src_val,
			 chan_val, yuv_val == TRUE ? "-x v4l=yuv422" : "", file_val,
			 vbit_val, abit_val, agn_val, deinter == TRUE ? "-I 5" : "",
			 prev_val == TRUE ? "-J preview" : "", split_val);
		if (duration != 0)
			fprintf (outFile, "&\n/tmp/gv4l-sleep.sh\n");
		else
			fprintf (outFile, "\n");

		fprintf (outFile,
			 "avimerge -o \"%s.new\" -i \"%s\" \"%s.append\" 2>/dev/null||echo Done > /tmp/gv4l.temp|exit\n",
			 file_val, file_val, file_val);
		fprintf (outFile, "rm \"%s\" \"%s.append\"\n", file_val, file_val);
		fprintf (outFile, "mv \"%s.new\" \"%s\"\n", file_val, file_val);
		fprintf (outFile, "echo Done > /tmp/gv4l.temp\n");

		fprintf (outFile, "echo -e '\n\n\'\n");
		fclose (outFile);

		if (duration != 0)
		{
			outFile = fopen ("/tmp/gv4l-sleep.sh", "w");
			fprintf (outFile, "#!/bin/bash\n");
			fprintf (outFile,
				 "sleep %dm\nkillall -2 transcode\nkillall -3 gv4l\nrm -rf /tmp/gv4l*\n",
				 duration);
			fclose (outFile);
		}

		//set the norm
		set_norm (vdev_val, norm_val);
		//run script
		system ("chmod +x /tmp/gv4l.sh");
		system ("chmod +x /tmp/gv4l-sleep.sh 2>/dev/null");
		system ("/tmp/gv4l.sh >>/tmp/tr-output 2>&1 &");
	}


	//change status
	style->text[GTK_STATE_NORMAL] = red;
	//	style->font = gdk_font_load("10x20");
	gtk_entry_set_text (GTK_ENTRY (status), "Recording...");
	gtk_widget_set_style (GTK_WIDGET (status), style);
	sleep (1);
	output_show ();

	//remove check file
	system ("rm -rf /tmp/trr");
	//create new check file if transcode is running
	system ("ps -A|grep transcode >/tmp/trr || rm -rf /tmp/trr");

	//reset status if transcode not running
	if (fopen ("/tmp/trr", "r") == NULL)
	{
		style0->text[GTK_STATE_NORMAL] = green;
		// style->font = gdk_font_load("10x20");
		gtk_entry_set_text (GTK_ENTRY (status), "Idle.");
		gtk_widget_set_style (GTK_WIDGET (status), style0);
	}
}


//call the record function from menu
void on_start_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_start_clicked ((GtkButton *) menuitem, menuitem);
}





//stop function
void on_stop_clicked (GtkButton * button, gpointer user_data)
{
	//lookup widgets
	GtkWidget *status = lookup_widget (GTK_WIDGET (button), "status");
	GtkStyle *style = gtk_style_new ();
	GdkColor green = { 0, 0, 35335, 0 };

	//stop transcode
	system ("killall -2 transcode 2>/dev/null || echo Done > /tmp/gv4l.temp");
	system ("killall -9 gv4l-sleep.sh 2>/dev/null");
	//wait until scripts are done
	do
		system ("sleep 0.5 2>/dev/null || sleep 0,5");
	while (fopen ("/tmp/gv4l.temp", "r") == NULL);
	system ("sleep 0.1 2>/dev/null || sleep 0,1");

	//remove temp files
	system ("rm -rf /tmp/gv4l*");
	system ("rm -rf /tmp/trr");
	output_show ();

	//reset status
	style->text[GTK_STATE_NORMAL] = green;
	//style->font = gdk_font_load("10x20");
	gtk_entry_set_text (GTK_ENTRY (status), "Idle.");
	gtk_widget_set_style (GTK_WIDGET (status), style);
}


//call stop function from menu
void on_stop_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_stop_clicked ((GtkButton *) menuitem, menuitem);
}




//show the transcode output window
void on_transcode_output_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	//create the window
	if (visible[1] == 0)
	{
		transcode_output = create_transcode_output ();
		gtk_widget_show (transcode_output);
		visible[1] = 1;
	}
	output_show ();
}


//read transcode output
void output_show ()
{
	GtkWidget *view;
	GtkTextIter start, end;
	GtkTextBuffer *buffer;
	FILE *inFile;
	const gchar *filename;
	gint bytes_read;
	gchar bufsize[BUFFER_SIZE];

	//GString *new_command;

	//see if window should be created
	if (visible[1] == 0)
		transcode_output = create_transcode_output ();
	view = lookup_widget (transcode_output, "textview");
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));

	//convert to utf-8 otherwise it won't work
	//system ("iconv -t=utf-8 /tmp/tr-output >/tmp/gv4l-tr-out-utf 2>/dev/null");
	inFile = fopen ("/tmp/tr-output", "r");
	if (inFile == NULL)
	{
		gtk_text_buffer_set_text (buffer, "", -1);
		return;
	}
	//clear the contents
	gtk_text_buffer_set_text (buffer, "", -1);
	//display the output
	for (;;)
	{
		bytes_read = fread (bufsize, sizeof (gchar), BUFFER_SIZE, inFile);

		if (bytes_read > 0)
		{
			gtk_text_buffer_get_end_iter (buffer, &end);
			gtk_text_buffer_insert (buffer, &end,
			g_locale_to_utf8 (bufsize, -1, NULL, NULL,
								NULL), bytes_read);
		}
		if (bytes_read != BUFFER_SIZE && (feof (inFile) || ferror (inFile)))
			break;
	}
	if (ferror (inFile))
	{
		fclose (inFile);
		gtk_text_buffer_get_bounds (buffer, &start, &end);
		gtk_text_buffer_delete (buffer, &start, &end);
		gtk_text_buffer_set_text (buffer, "Unable to load file", -1);
		return;
	}

	fclose (inFile);
}





//close the output window
gboolean on_transcode_output_delete_event (GtkWidget * widget,
											GdkEvent * event, gpointer user_data)
{
	//make sure we have the window created and visible flag is disabled
	transcode_output = create_transcode_output ();
	visible[1] = 0;
	return FALSE;
}


//close the output window
void on_output_hide_clicked (GtkButton * button, gpointer user_data)
{
	//destroy the window and re-create for later use
	gtk_widget_destroy (transcode_output);
	on_transcode_output_delete_event ((GtkWidget *) button, NULL, button);
}





//clear the output
void
on_clear_output_clicked (GtkButton * button, gpointer user_data)
{
	//remove logs and clear the window
	system ("rm -rf /tmp/tr-out*");
	output_show ();
}
